@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: CmdExec.h,v 1.76 2011/06/10 09:36:43 lav Exp $ */
-
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
 
@@ -54,6 +52,8 @@ public:
 
    virtual void Fg() {}
    virtual void Bg() {}
+
+   virtual bool IsInteractive() const { return false; }
 };
 
 extern CmdFeeder *lftp_feeder;	 // feeder to use after 'lftp' command
@@ -165,6 +165,9 @@ public:
    void Exit(int);
    void AtExit();
    void AtExitBg();
+   void AtExitFg();
+   void AtBackground();
+   void AtTerminate();
    void EmptyCmds();
    bool WriteCmds(int fd) const;
    bool ReadCmds(int fd); // does not clear queue before reading (appends)
@@ -188,6 +191,7 @@ public:
    const char *MakePrompt();
 
    bool interactive;
+   bool show_status;
    bool top_level;
    bool verbose;
    bool auto_terminate_in_bg;
@@ -220,10 +224,12 @@ public:
    void top_vfprintf(FILE *file,const char *f,va_list v);
 
    void SetInteractive(bool i);
+   void SetInteractive();
    void SetTopLevel()
       {
 	 top_level=true;
 	 Reconfig(0);
+	 SetInteractive();
       }
    void SetStatusLine(StatusLine *s) { status_line=s; }
    void SetAutoTerminateInBackground(bool b) { auto_terminate_in_bg=b; }
