@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileCopy.cc,v 1.162 2011/04/22 15:43:11 lav Exp $ */
-
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
     2) tries to read some data at seek_pos, sets pos to position of Get (get).
@@ -254,7 +251,7 @@ int FileCopy::Do()
 	    return MOVED;
 	 }
       }
-      if(put->Size()>max_buf)
+      if(put->IsFull())
 	 get->Suspend(); // stall the get.
       get->Get(&b,&s);
       if(b==0) // eof
@@ -415,9 +412,11 @@ FileCopy::FileCopy(FileCopyPeer *s,FileCopyPeer *d,bool c)
    rate_for_eta(new Speedometer("xfer:eta-period"))
 {
    set_state(INITIAL);
-   max_buf=buffer_size.Query(0);
+   int max_buf=buffer_size.Query(0);
    if(max_buf<1)
       max_buf=1;
+   s->SetMaxBuffered(max_buf);
+   d->SetMaxBuffered(max_buf);
    put_buf=0;
    put_eof_pos=0;
    bytes_count=0;
@@ -441,6 +440,7 @@ FileCopy *FileCopy::New(FileCopyPeer *s,FileCopyPeer *d,bool c)
 }
 void FileCopy::SuspendInternal()
 {
+   super::SuspendInternal();
    if(get) get->SuspendSlave();
    if(put) put->SuspendSlave();
 }
@@ -448,6 +448,7 @@ void FileCopy::ResumeInternal()
 {
    if(get) get->ResumeSlave();
    if(put) put->ResumeSlave();
+   super::ResumeInternal();
 }
 void FileCopy::Fg()
 {
@@ -631,10 +632,11 @@ void FileCopy::LogTransfer()
    {
       const char *fname = ResMgr::Query("xfer:log-file", 0);
       if(!fname || !*fname)
-	 fname = dir_file(get_lftp_home(),"transfer_log");
+	 fname = dir_file(get_lftp_data_dir(),"transfer_log");
       int fd = open(fname,O_WRONLY|O_APPEND|O_CREAT,0600);
       if(fd==-1)
 	 return;
+      fcntl(fd,F_SETFD,FD_CLOEXEC);
       transfer_log=new Log;
       transfer_log->SetOutput(fd,true);
       transfer_log->ShowNothing();
@@ -647,7 +649,7 @@ void FileCopy::LogTransfer()
    transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",
       url::remove_password(src),url::remove_password(dst),
       (long long)GetRangeStart(),range_limit,
-      Speedometer::GetStr(GetBytesCount()/GetTimeSpent()).get());
+      Speedometer::GetStrProper(GetBytesCount()/GetTimeSpent()).get());
 }
 
 void FileCopy::SetRange(off_t s,off_t lim)
@@ -727,6 +729,7 @@ FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
    want_date=false;
    start_transfer=true;
    size=NO_SIZE_YET;
+   date=NO_DATE_YET;
    e_size=NO_SIZE;
    seek_pos=0;
    can_seek=false;
@@ -783,14 +786,19 @@ int FileCopyPeerFA::Do()
       return m;
    }
 
-   if(want_size && size==NO_SIZE_YET && (mode==PUT || !start_transfer))
+   if((want_size && size==NO_SIZE_YET && (mode==PUT || !start_transfer))
+   || (want_date && date==NO_DATE_YET))
    {
       if(session->IsClosed())
       {
-	 info.file=file;
-	 info.get_size=true;
-	 info.get_time=want_date;
-	 session->GetInfoArray(&info,1);
+	 FileInfo *fi=new FileInfo(file);
+	 if(want_size)
+	    fi->Need(fi->SIZE);
+	 if(want_date)
+	    fi->Need(fi->DATE);
+	 info.Empty();
+	 info.Add(fi);
+	 session->GetInfoArray(&info);
 	 m=MOVED;
       }
       res=session->Done();
@@ -802,8 +810,11 @@ int FileCopyPeerFA::Do()
 	 SetSize(NO_SIZE);
 	 return MOVED;
       }
-      SetSize(info.size);
-      SetDate(info.time);
+      FileInfo *fi=info[0];
+      if(want_size)
+	 SetSize(fi->size);
+      if(want_date)
+	 SetDate(fi->date);
       session->Close();
       return MOVED;
    }
@@ -915,9 +926,11 @@ void FileCopyPeerFA::SuspendInternal()
       return;
    if(session->IsOpen())
       session->SuspendSlave();
+   super::SuspendInternal();
 }
 void FileCopyPeerFA::ResumeInternal()
 {
+   super::ResumeInternal();
    session->ResumeSlave();
 }
 
@@ -1562,6 +1575,10 @@ int FileCopyPeerFDStream::Get_LL(int len)
       {
 	 SetDate(st.st_mtime);
 	 SetSize(st.st_size);
+#ifndef NATIVE_CRLF
+	 if(ascii)
+	    SetSize(NO_SIZE);
+#endif
       }
    }
 
@@ -1814,6 +1831,21 @@ int FileCopyPeerDirList::Do()
    return MOVED;
 }
 
+// FileCopyPeerMemory
+int FileCopyPeerMemory::Do()
+{
+   int m=STALL;
+   if(mode==PUT) {
+      max_buf=max_size+1;
+      if(Size()>max_size) {
+	 SetError("buffer limit exceeded");
+	 broken=true;
+	 return MOVED;
+      }
+   }
+   return m;
+}
+
 // FileVerificator
 void FileVerificator::Init0()
 {
